#include <SDL.h>
#include <GL/gl.h>
#include "graphics.hpp"

namespace oiran { namespace linda {


	void
	graphics_t::clear()
	{
		glClear(GL_COLOR_BUFFER_BIT);
	}

	void
	graphics_t::flip()
	{
		SDL_GL_SwapBuffers();
	}

	void
	graphics_t::clear_color(double r, double g, double b, double a)
	{
		glClearColor(r, g, b, a);
	}

	void
	graphics_t::clear_color(unsigned int hex, double alpha)
	{
		unsigned int r = (hex >> 16) & 0xff;
		unsigned int g = (hex >> 8) & 0xff;
		unsigned int b = hex & 0xff;

		clear_color(r / 255.0, g / 255.0, b / 255.0, alpha);
	}

	void
	graphics_t::color(double r, double g, double b, double a)
	{
		glColor4d(r, g, b, a);
	}

	void
	graphics_t::color(unsigned int hex, double alpha)
	{
		unsigned int r = (hex >> 16) & 0xff;
		unsigned int g = (hex >> 8) & 0xff;
		unsigned int b = hex & 0xff;

		color(r / 255.0, g / 255.0, b / 255.0, alpha);
	}

	void
	graphics_t::point(double x, double y)
	{
		glBegin(GL_POINTS);
		glVertex2d(x, y);
		glEnd();
	}

	void
	graphics_t::line(double x1, double y1, double x2, double y2)
	{
		glBegin(GL_LINES);
		glVertex2d(x1, y1);
		glVertex2d(x2, y2);
		glEnd();
	}

	void
	graphics_t::triangle(double x1, double y1,
				double x2, double y2,
				double x3, double y3)
	{
		glBegin(GL_TRIANGLES);
		glVertex2d(x1, y1);
		glVertex2d(x2, y2);
		glVertex2d(x3, y3);
		glEnd();
	}

	void
	graphics_t::rect(double x, double y, double w, double h)
	{
		glRectd(x, y, x + w, y + h);
	}

	void
	graphics_t::quad(double x1, double y1,
				double x2, double y2,
				double x3, double y3,
				double x4, double y4)
	{
		glBegin(GL_QUADS);
		glVertex2d(x1, y1);
		glVertex2d(x2, y2);
		glVertex2d(x3, y3);
		glVertex2d(x4, y4);
		glEnd();
	}

	void
	graphics_t::smooth()
	{
		glEnable(GL_POINT_SMOOTH);
		glEnable(GL_LINE_SMOOTH);
	}

	void
	graphics_t::no_smooth()
	{
		glDisable(GL_POINT_SMOOTH);
		glDisable(GL_LINE_SMOOTH);
	}

	void
	graphics_t::stroke()
	{
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	}

	void
	graphics_t::fill()
	{
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	}

	void
	graphics_t::point_size(double value)
	{
		glPointSize(value);
	}

	void
	graphics_t::push_matrix()
	{
		glPushMatrix();
	}

	void
	graphics_t::pop_matrix()
	{
		glPopMatrix();
	}

	void
	graphics_t::rotate(double angle)
	{
		glRotated(angle, 0, 0, 1);
	}

	void
	graphics_t::translate(double x, double y)
	{
		glTranslated(x, y, 0);
	}

	void
	graphics_t::scale(double value)
	{
		glScaled(value, value, 1);
	}

	void
	graphics_t::scale(double x, double y)
	{
		glScaled(x, y, 1);
	}


}} // namespace oiran::linda.
