#include <algorithm>
#include "defs.hpp"
#include "field.hpp"
#include "mino.hpp"


const int field_t::line_clear_delay = framerate / 4;


field_t::field_t() :
		M_mino(0),
		M_state(wait),
		M_state_count(0),
		M_score(0)
{
	std::fill(M_blocks, M_blocks + field_width * field_height, 0);
}

void
field_t::init(mino_t* mino)
{
	M_mino = mino;

	M_state = wait;
	M_state_count = 0;

	std::fill(M_blocks, M_blocks + field_width * field_height, 0);
	M_score = 0;
}

void
field_t::update()
{
	if(M_state == line_clear)
	{
		if(M_state_count == 0)
			_check_lines();
		else if(++M_state_count > line_clear_delay)
			_clear_lines();
	}
}

void
field_t::draw(oiran::linda::graphics_t& g)
{
	g.push_matrix();
	g.translate(10, 30);

	for(int i = 1; i < field_height; ++i)
		for(int j = 0; j < field_width; ++j)
			if(M_blocks[i * field_width + j])
			{
				g.color(colors[M_blocks[i * field_width + j]]);
				g.rect(j * block_size, i * block_size,
						block_size, block_size);
			}

	g.color(colors[0]);
	g.stroke();
	g.rect(-1, block_size,
			field_width * block_size + 1, (field_height - 1) * block_size + 1);
	g.fill();

	g.pop_matrix();
}

bool
field_t::collision_detection(int x, int y, const int* mino)
{
	for(int i = 0; i < mino_size; ++i)
		for(int j = 0; j < mino_size; ++j)
			if(*(mino + i * mino_size + j))
				if(x + j < 0 || x + j >= field_width ||
						y + i < 0 || y + i >= field_height ||
						M_blocks[(y + i) * field_width + x + j])
					return true;
	return false;
}

void
field_t::lock(int x, int y, const int* mino)
{
	for(int i = 0; i < mino_size; ++i)
		for(int j = 0; j < mino_size; ++j)
			if(*(mino + i * mino_size + j))
				M_blocks[(y + i) * field_width + x + j] =
						*(mino + i * mino_size + j);

	M_state = line_clear;
	M_state_count = 0;
}

void
field_t::_check_lines()
{
	int count = 0;
	for(int i = 0; i < field_height; ++i)
		if(std::count(M_blocks + i * field_width,
					M_blocks + (i + 1) * field_width, 0) == 0)
		{
			std::fill(M_blocks + i * field_width,
					M_blocks + (i + 1) * field_width, 0);
			++count;
		}

	if(!count)
	{
		M_state = wait;
		M_mino->entry();
	}
	else
	{
		switch(count)
		{
		case 1:	M_score += 50; break;
		case 2:	M_score += 200; break;
		case 3:	M_score += 450; break;
		case 4:	M_score += 1000; break;
		}
		M_mino->level_up(count);
		M_state_count = 1;
	}
}

void
field_t::_clear_lines()
{
	for(int i = field_height - 1; i > 0; --i)
		if(std::count(M_blocks + i * field_width,
					M_blocks + (i + 1) * field_width, 0) == field_width)
			for(int j = i - 1; j >= 0; --j)
				if(std::count(M_blocks + j * field_width,
							M_blocks + (j + 1) * field_width, 0) != field_width)
				{
					std::swap_ranges(M_blocks + i * field_width,
							M_blocks + (i + 1) * field_width,
							M_blocks + j * field_width);
					break;
				}

	M_state = wait;
	M_mino->entry();
}
