#include <algorithm>
#include <functional>
#include <oiran/random.hpp>
#include "defs.hpp"
#include "field.hpp"
#include "mino.hpp"


namespace {


	class rand_t : public std::unary_function<int, int>
	{
	public:
		int operator()(int max)
		{ return static_cast<int>(oiran::random(max)); }
	};


	const int mino_data_size = mino_size * mino_size + 2;

	const int mino_data[] = {
		// t_type.
		0, 0, 0, 0,
		1, 1, 1, 0,
		0, 1, 0, 0,
		0, 0, 0, 0,
		1, 3,

		0, 1, 0, 0,
		0, 1, 1, 0,
		0, 1, 0, 0,
		0, 0, 0, 0,
		2, 0,

		0, 0, 0, 0,
		0, 1, 0, 0,
		1, 1, 1, 0,
		0, 0, 0, 0,
		3, 1,

		0, 1, 0, 0,
		1, 1, 0, 0,
		0, 1, 0, 0,
		0, 0, 0, 0,
		0, 2,

		// j_type.
		0, 0, 0, 0,
		2, 2, 2, 0,
		0, 0, 2, 0,
		0, 0, 0, 0,
		5, 7,

		0, 2, 2, 0,
		0, 2, 0, 0,
		0, 2, 0, 0,
		0, 0, 0, 0,
		6, 4,

		0, 0, 0, 0,
		2, 0, 0, 0,
		2, 2, 2, 0,
		0, 0, 0, 0,
		7, 5,

		0, 2, 0, 0,
		0, 2, 0, 0,
		2, 2, 0, 0,
		0, 0, 0, 0,
		4, 6,

		// l_type.
		0, 0, 0, 0,
		3, 3, 3, 0,
		3, 0, 0, 0,
		0, 0, 0, 0,
		9, 11,

		0, 3, 0, 0,
		0, 3, 0, 0,
		0, 3, 3, 0,
		0, 0, 0, 0,
		10, 8,

		0, 0, 0, 0,
		0, 0, 3, 0,
		3, 3, 3, 0,
		0, 0, 0, 0,
		11, 9,

		3, 3, 0, 0,
		0, 3, 0, 0,
		0, 3, 0, 0,
		0, 0, 0, 0,
		8, 10,

		// z_type.
		0, 0, 0, 0,
		4, 4, 0, 0,
		0, 4, 4, 0,
		0, 0, 0, 0,
		13, 13,

		0, 0, 4, 0,
		0, 4, 4, 0,
		0, 4, 0, 0,
		0, 0, 0, 0,
		12, 12,

		// s_type.
		0, 0, 0, 0,
		0, 5, 5, 0,
		5, 5, 0, 0,
		0, 0, 0, 0,
		15, 15,

		5, 0, 0, 0,
		5, 5, 0, 0,
		0, 5, 0, 0,
		0, 0, 0, 0,
		14, 14,

		// i_type.
		0, 0, 0, 0,
		6, 6, 6, 6,
		0, 0, 0, 0,
		0, 0, 0, 0,
		17, 17,

		0, 0, 6, 0,
		0, 0, 6, 0,
		0, 0, 6, 0,
		0, 0, 6, 0,
		16, 16,

		// o_type.
		0, 0, 0, 0,
		0, 7, 7, 0,
		0, 7, 7, 0,
		0, 0, 0, 0,
		18, 18,
	};

	const int mino_data_index[] = {
		0,	// t_type.
		4,	// j_type.
		8,	// l_type.
		12,	// z_type.
		14,	// s_type.
		16,	// i_type.
		18,	// o_type.
	};


} // unnamed namespace.


next_mino_t::next_mino_t() :
		M_index(0)
{
	init();
}

void
next_mino_t::init()
{
	for(int i = 0; i < set_size; ++i)
		for(int j = 0; j < number_of_minos; ++j)
			M_minos[i * number_of_minos + j] = j + t_type;

	rand_t rand;
	std::random_shuffle(M_minos, M_minos + number_of_minos * set_size, rand);

	M_index = 0;
}

void
next_mino_t::pop()
{
	if(++M_index >= number_of_minos * set_size)
		init();
}

void
next_mino_t::draw(oiran::linda::graphics_t& g)
{
	g.push_matrix();
	g.translate(40, 0);

	const int* index = mino_data + mino_data_index[top()] * mino_data_size;
	for(int i = 0; i < mino_size; ++i)
		for(int j = 0; j < mino_size; ++j)
			if(*(index + i * mino_size + j))
			{
				g.color(colors[*(index + i * mino_size + j)]);
				g.rect(j * block_size, i * block_size,
						block_size, block_size);
			}

	g.pop_matrix();
}


const int mino_t::entry_x = 3;
const int mino_t::entry_y = -1;
const int mino_t::are = framerate / 2;
const int mino_t::lock_delay = framerate / 2;
const int mino_t::auto_shift_delay = framerate / 4;
const int mino_t::max_level = 999;


mino_t::mino_t() :
		M_input(0),
		M_state(wait),
		M_state_count(0),
		M_blocks(0),
		M_x(entry_x),
		M_y(entry_y),
		M_level(0),
		M_gravity(1),
		M_gravity_delay(framerate),
		M_gravity_count(0),
		M_lock_count(0)
{
	std::fill(M_next_indexs, M_next_indexs + next_index_size, 0);
}

void
mino_t::init(oiran::linda::button_input_t* input, field_t* field)
{
	M_input = input;
	M_field = field;

	M_next_mino.init();

	M_state = wait;
	M_state_count = 0;

	M_blocks = mino_data + mino_data_index[M_next_mino.top()] * mino_data_size;
	for(int i = 0; i < next_index_size; ++i)
		M_next_indexs[i] = *(M_blocks + mino_size * mino_size + i);
	M_next_mino.pop();

	M_x = entry_x;
	M_y = entry_y;
	M_level = 0;
	M_gravity = 1;
	M_gravity_delay = framerate;
	M_gravity_count = 0;
	M_lock_count = 0;
}

void
mino_t::update()
{
	switch(M_state)
	{
	case entry_wait:
		if(++M_state_count > are)
			_entry();
		break;
	case on_field:
		{
			int left = M_input->get(M_input->left);
			int right = M_input->get(M_input->right);
			if(left == 1 || left > auto_shift_delay)
				_shift(-1);
			if(right == 1 || right > auto_shift_delay)
				_shift(1);
		}

		if(M_input->is_pushed(M_input->a_button))
			_rotate(left);
		if(M_input->is_pushed(M_input->b_button))
			_rotate(right);

		if(M_input->is_pressed(M_input->down))
			_soft_drop();

		if(M_input->is_pressed(M_input->up))
			_hard_drop();

		_down_by_gravity();
		break;
	default:
		break;
	}
}

void
mino_t::draw(oiran::linda::graphics_t& g)
{
	if(M_blocks && M_state == on_field)
	{
		g.push_matrix();
		g.translate(10, 30);

		for(int i = 0; i < mino_size; ++i)
			for(int j = 0; j < mino_size; ++j)
				if(*(M_blocks + i * mino_size + j) && M_y + i > 0)
				{
					g.color(colors[*(M_blocks + i * mino_size + j)],
							1 - 0.5 * M_lock_count / lock_delay);
					g.rect((M_x + j) * block_size, (M_y + i) * block_size,
							block_size, block_size);
				}

		g.pop_matrix();
	}

	M_next_mino.draw(g);
}

void
mino_t::entry()
{
	if(M_state == game_over || M_state == game_clear)
		return;

	M_state = entry_wait;
	M_state_count = 0;
}

void
mino_t::level_up(int value)
{
	M_level = std::min(M_level + value, max_level);
	if(M_level == max_level)
		M_state = game_clear;

	if(M_level < 300)
		M_gravity = 1;
	else if(M_level <= 333)
		M_gravity = 2;
	else if(M_level <= 366)
		M_gravity = 3;
	else if(M_level < 400)
		M_gravity = 4;
	else if(M_level <= 433)
		M_gravity = 5;
	else if(M_level <= 466)
		M_gravity = 4;
	else if(M_level < 500)
		M_gravity = 3;
	else
		M_gravity = 20;

	if(M_level < 50)
		M_gravity_delay = 128;
	else if(M_level < 100)
		M_gravity_delay = 64;
	else if(M_level < 125)
		M_gravity_delay = 32;
	else if(M_level < 150)
		M_gravity_delay = 16;
	else if(M_level < 175)
		M_gravity_delay = 8;
	else if(M_level < 200)
		M_gravity_delay = 4;
	else if(M_level < 225)
		M_gravity_delay = 8;
	else if(M_level < 250)
		M_gravity_delay = 4;
	else if(M_level < 275)
		M_gravity_delay = 2;
	else
		M_gravity_delay = 1;
}

void
mino_t::_entry()
{
	M_x = entry_x;
	M_y = entry_y;
	M_gravity_count = 0;
	M_lock_count = 0;

	M_blocks = mino_data + mino_data_index[M_next_mino.top()] * mino_data_size;
	for(int i = 0; i < next_index_size; ++i)
		M_next_indexs[i] = *(M_blocks + mino_size * mino_size + i);
	M_next_mino.pop();

	if(M_input->is_pressed(M_input->a_button))
	{
		M_blocks = mino_data + M_next_indexs[left] * mino_data_size;
		for(int i = 0; i < next_index_size; ++i)
			M_next_indexs[i] = *(M_blocks + mino_size * mino_size + i);
	}
	if(M_input->is_pressed(M_input->b_button))
	{
		M_blocks = mino_data + M_next_indexs[right] * mino_data_size;
		for(int i = 0; i < next_index_size; ++i)
			M_next_indexs[i] = *(M_blocks + mino_size * mino_size + i);
	}

	if(M_field->collision_detection(M_x, M_y + 1, M_blocks))
	{
		M_field->lock(M_x, M_y + 1, M_blocks);
		M_state = game_over;
		return;
	}

	if(M_level % 100 != 99 && M_level != max_level - 1)
		level_up(1);

	M_state = on_field;

	_down();
}

void
mino_t::_shift(int x)
{
	if(!M_field->collision_detection(M_x + x, M_y, M_blocks))
		M_x += x;
}

void
mino_t::_rotate(int next_index)
{
	const int* next = mino_data + M_next_indexs[next_index] * mino_data_size;

	int x = 0;
	bool can_rotate = !M_field->collision_detection(M_x, M_y, next);

	if(!can_rotate)
	{
		x = 1;
		can_rotate = !M_field->collision_detection(M_x + x, M_y, next);
	}

	if(!can_rotate)
	{
		x = -1;
		can_rotate = !M_field->collision_detection(M_x + x, M_y, next);
	}

	if(can_rotate)
	{
		M_blocks = next;
		for(int i = 0; i < next_index_size; ++i)
			M_next_indexs[i] = *(M_blocks + mino_size * mino_size + i);
		M_x += x;
	}
}

void
mino_t::_soft_drop()
{
	if(!M_field->collision_detection(M_x, M_y + 1, M_blocks))
		M_y += 1;
}

void
mino_t::_hard_drop()
{
	if(M_lock_count)
		return;

	int y = 0;
	for(; !M_field->collision_detection(M_x, M_y + y + 1, M_blocks); ++y);
	M_y += y;

	M_lock_count = 1;
}

void
mino_t::_down()
{
	int y = 0;
	for(; y < M_gravity &&
			!M_field->collision_detection(M_x, M_y + y + 1, M_blocks); ++y);
	M_y += y;

	if(y != M_gravity)
		M_lock_count = 1;
	else
		M_lock_count = 0;

	M_gravity_count = 0;
}

void
mino_t::_lock()
{
	M_field->lock(M_x, M_y, M_blocks);
	M_state = wait;
}

void
mino_t::_down_by_gravity()
{
	if(M_lock_count)
	{
		if(!M_field->collision_detection(M_x, M_y + 1, M_blocks))
			_down();
		else if(++M_lock_count > lock_delay)
			_lock();
		else if(M_input->is_pushed(M_input->down))
			_lock();
	}
	else
	{
		if(M_field->collision_detection(M_x, M_y + 1, M_blocks) &&
				M_input->is_pushed(M_input->down))
			_lock();
		else if(++M_gravity_count > M_gravity_delay)
			_down();
	}
}
