#ifndef SCENE_HPP
#define SCENE_HPP

#include <fstream>
#include <oiran/linda/button_input.hpp>
#include <oiran/linda/scene.hpp>
#include "ttrs.hpp"
#include "ttrs_replay.hpp"


enum {
	scene_title,
	scene_game,
	scene_ranking,
	scene_replay
};


class scene_manager_t;


class scene_title_t : public oiran::linda::scene_t
{
public:
	enum {
		demo,
		on_select
	};

	scene_title_t(scene_manager_t* manager);

	void update();
	void draw(oiran::linda::graphics_t& g);

private:
	scene_manager_t* M_manager;

	ttrs_replay_t M_replay;

	int M_select;
	double M_alphas[2];

	int M_state;
	int M_state_count;

	double M_title_alpha;
	double M_menu_alpha;
	double M_ttrs_alpha;
};


class scene_game_t : public oiran::linda::scene_t
{
public:
	scene_game_t();

	void update();
	void draw(oiran::linda::graphics_t& g);

private:
	ttrs_t M_ttrs;
	std::ofstream M_out;
};


class scene_ranking_t : public oiran::linda::scene_t
{
public:
	scene_ranking_t(scene_manager_t* manager);
	~scene_ranking_t();

	void update();
	void draw(oiran::linda::graphics_t& g);

private:
	void _init_replay();

private:
	scene_manager_t* M_manager;

	ttrs_replay_t M_replay;

	int M_select;
	double M_alphas[ranking_size];
};


class scene_replay_t : public oiran::linda::scene_t
{
public:
	static const int max_speed;

	scene_replay_t(int index);

	void update();
	void draw(oiran::linda::graphics_t& g);

private:
	ttrs_replay_t M_replay;
	int M_speed;
};


class scene_manager_t : public oiran::linda::scene_manager_t
{
protected:
	oiran::linda::scene_t* create(int id);
};


#endif // SCENE_HPP
