#include <algorithm>
#include <fstream>
#include <sstream>
#include "defs.hpp"
#include "ranking.hpp"
#include "resource.hpp"
#include "scene.hpp"


scene_ranking_t::scene_ranking_t(scene_manager_t* manager) :
		M_manager(manager),
		M_select(0)
{
	std::fill(M_alphas, M_alphas + ranking_size, 0.5);
	M_alphas[0] = 1;

	_init_replay();
}

scene_ranking_t::~scene_ranking_t()
{
	ranking_t& ranking = resource_t::instance()->ranking;

	ranking.save();
	ranking.clear();
}

void
scene_ranking_t::update()
{
	oiran::linda::joystick_t& stick = resource_t::instance()->stick;

	stick.update();

	if(M_select > 0 && stick.is_pushed(stick.up))
	{
		--M_select;
		M_alphas[M_select] = 1;
		_init_replay();
	}
	if(M_select < ranking_size - 1 && stick.is_pushed(stick.down))
	{
		++M_select;
		M_alphas[M_select] = 1;
		_init_replay();
	}

	if(stick.is_pushed(stick.a_button) && M_replay.can_replay())
	{
		M_manager->push(scene_ranking);
		M_manager->push(scene_replay + M_select);
		valid(false);
	}

	if(stick.is_pushed(stick.b_button))
		valid(false);

	if(M_replay.can_replay())
	{
		for(int i = 0; i < 2; ++i)
			M_replay.update();
		if(M_replay.is_end())
			_init_replay();
	}

	for(int i = 0; i < ranking_size; ++i)
		if(M_select != i)
			M_alphas[i] = std::max(0.5, M_alphas[i] - 0.05);
}

void
scene_ranking_t::draw(oiran::linda::graphics_t& g)
{
	oiran::linda::simple_font_t& font = resource_t::instance()->font;
	ranking_t& ranking = resource_t::instance()->ranking;

	g.color(colors[0]);
	font.size(24, 32);
	font.print(1, 1, "RANKING");

	font.size(12, 24);
	for(int i = 0; i < ranking_size; ++i)
	{
		g.color(colors[0], M_alphas[i]);
		font.print(10, 40 + i * 25, i + 1);
		if(i == ranking.update_index())
			g.color(colors[6], M_alphas[i]);
		else
			g.color(colors[0], M_alphas[i]);
		font.print(36, 40 + i * 25, ranking.score(i));
	}

	g.push_matrix();
	g.translate(width / 2 - 2, height / 2 - 13);
	g.scale(0.5);
	if(M_replay.can_replay())
		M_replay.draw(g);
	g.color(colors[0]);
	g.stroke();
	g.rect(0, 0, width, height);
	g.fill();
	g.pop_matrix();

	g.color(colors[0]);
	font.size(6, 12);
	font.print(1, height - 13, "push b button to return");
}

void
scene_ranking_t::_init_replay()
{
	std::ostringstream oss;
	oss << M_select << ".dat";

	M_replay.init(oss.str().c_str());
}
